import 'dart:ui';
import 'package:flutter/material.dart';
import 'Module/DatePicker/date_card_view.dart';
import 'TimePicker/time_view_card.dart';
import 'data_list.dart';
import 'shared_class.dart';

// ignore: must_be_immutable
class BaseView extends StatefulWidget {
  final Color textColor;
  final Color backColor;
  final Color selectionColor;
  final Color unSelectedBackColor;
  final DateTime snapData;
  final double containerHeight;
  final double containerWidth;
  final double width;
  final double height;
  final double radius;
  final int timeValue;
  final bool is24Hours;
  bool hideSingle;
  final double expandWidth;
  final Function dateCallBack;
  final Function timeCallBack;
  final typesofCalendar calendarType;
  BaseView(
      this.textColor,
      this.backColor,
      this.selectionColor,
      this.unSelectedBackColor,
      this.snapData,
      this.containerHeight,
      this.containerWidth,
      this.width,
      this.height,
      this.radius,
      this.timeValue,
      this.hideSingle,
      this.expandWidth,
      this.dateCallBack,
      this.timeCallBack,
      this.calendarType,this.is24Hours);

  @override
  _BaseViewState createState() => _BaseViewState();
}

class _BaseViewState extends State<BaseView> {
  GlobalKey singleContainerKey = GlobalKey();
  GlobalKey multiContainerKey = GlobalKey();
  double _x, _y;
  bool isVisible = true;
  Size kCardSize;
  bool showSingleWidget = true;


  //GET THE SIZE OF THE CONTAINER
  Size _getSize() {
    final RenderBox renderBoxRed =
    singleContainerKey.currentContext.findRenderObject();
    return renderBoxRed.size;
  }
//GET THE OFFSET OF THE PLACED WIDGET

  void _getOffset(GlobalKey key) {
    RenderBox box = key.currentContext.findRenderObject();
    Offset position = box.localToGlobal(Offset.zero);
    _x = position.dx;
    _y = position.dy;
  }
  //CARD UI (A SINGLE CARD) TO REPRESNT THE WIDGET
  Widget _singleCard() {
    return Container(
        decoration: BoxDecoration(
          borderRadius: const BorderRadius.only(
            topLeft: Radius.circular(5),
            topRight: Radius.circular(5),
            bottomLeft: Radius.circular(5),
            bottomRight: Radius.circular(5),
          ),
        ),
        width: widget.containerWidth,
        child: widget.calendarType == typesofCalendar.days
            ? DateCardView(widget.textColor, widget.backColor, widget.snapData,
            widget.width, widget.height)
            : TimerCardView(widget.textColor, widget.backColor, widget.timeValue == 0 ? DateTime.now().hour:widget.timeValue, widget.width,
            widget.height));
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
        onTap: () {
          setState(() {
            widget.hideSingle = true;
          });
          _getOffset(singleContainerKey);
          kCardSize = _getSize();
          showGeneralDialog(
            barrierDismissible: true,
            barrierLabel: '',
            barrierColor: Colors.transparent,
            context: context,
            transitionBuilder: (context1, a1, a2, child) {
              final curvedValue = Curves.linear.transform(a1.value);
              return ClipOval(
                clipper: WidgetClipper(kCardSize, context, curvedValue, _x, _y,
                    widget.expandWidth),
                child: Transform(
                  transform: Matrix4.translationValues(0.0, curvedValue, 0.0),
                  child: Column(
                    children: [
                      Container(
                        color: Colors.transparent,
                        margin: EdgeInsets.only(
                            left: 0, top: _y-2, bottom: 0.0, right: 0.0),
                        height: kCardSize.height+2,
                        width: MediaQuery.of(context).size.width,
                        child: Material(
                          shape: RoundedRectangleBorder(
                              borderRadius:
                              BorderRadius.all(Radius.circular(5))),
                          child: Opacity(
                            opacity: 1.0,
                            child: Stack(
                              children: [
                                Column(
                                  mainAxisAlignment: MainAxisAlignment.center,
                                  crossAxisAlignment: CrossAxisAlignment.center,
                                  children: <Widget>[
                                    Stack(
                                      alignment: AlignmentDirectional.center,
                                      children: [
                                        DataList(
                                            Colors.transparent,
                                            Colors.transparent,
                                            widget.containerHeight,
                                            MediaQuery.of(context).size.width,
                                            widget.height,
                                            widget.width,
                                            10,
                                            widget.textColor,
                                            widget.unSelectedBackColor,
                                            true,
                                            widget.snapData,widget.timeValue,
                                            widget.dateCallBack,
                                            widget.timeCallBack,
                                            widget.calendarType,widget.is24Hours,_x),

                                      ],
                                    )
                                  ],
                                ),
                              ],
                            ),
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              );
            },
            transitionDuration: Duration(milliseconds: 4000),
            pageBuilder: (BuildContext context, anim1, anim2) {},
          );
        },
        child: Container(
            key: singleContainerKey,
            child: Visibility(
                child: widget.hideSingle ? _singleCard() : Container())));
  }
}
//OVAL CLIPPER FOR THE ANIMATION

class WidgetClipper extends CustomClipper<Rect> {
  Size widgetSize;
  double cuverdValue;
  BuildContext context;
  double x, y;
  double expandWidth;

  WidgetClipper(this.widgetSize, this.context, this.cuverdValue, this.x, this.y,
      this.expandWidth);

  Rect getClip(Size size) {
    return Rect.fromCircle(
        center: Offset(x + widgetSize.width / 2, y + widgetSize.height / 2),
        radius: lerpDouble(
          widgetSize.width,
          expandWidth,
          cuverdValue,
        ));
  }

  @override
  bool shouldReclip(covariant CustomClipper<Rect> oldClipper) {
    return true;
  }
}
