import 'dart:ui';

import 'package:custom_date_time_picker/shared_class.dart';
import 'package:flutter/material.dart';
import 'package:infinite_listview/infinite_listview.dart';

import 'Module/DatePicker/date_card_view.dart';
import 'TimePicker/time_view_card.dart';

class DataList extends StatefulWidget {
  final Color textColor;
  final Color listBackColor;
  final double height;
  final double width;
  final double containerHeight;
  final double containerWidth;
  final double cardElevation;
  final Color selectedTextColor;
  final Color selectedBackColor;
  final bool showTitle;
  final DateTime selectedDate;
  final Function clickCallBack;
  final Function timeCallBack;
  final bool is24Hours;
  final int selectTime;
  final typesofCalendar typeOfCalendar;
  final bool isItemSelected = false;
  final double initalOffset;
  DataList(
      this.textColor,
      this.listBackColor,
      this.containerHeight,
      this.containerWidth,
      this.height,
      this.width,
      this.cardElevation,
      this.selectedTextColor,
      this.selectedBackColor,
      this.showTitle,
      this.selectedDate,
      this.selectTime,
      this.clickCallBack,
      this.timeCallBack,
      this.typeOfCalendar,
      this.is24Hours,this.initalOffset);

  @override
  _DataListState createState() => _DataListState();
}

class _DataListState extends State<DataList> {
  InfiniteScrollController _infiniteController;
  InfiniteScrollController controllerA;
  int popUpIndex = 0;
  double offsetTemp;
  int selectedSetIndex = 0;
  int difference;
  var finalValue;

  @override
  void initState() {
    //INITIAL OFFSET OF LIST VIEW AS PER THE CURRENT DATE OR LAST SELECTED DATE
    if (widget.typeOfCalendar == typesofCalendar.days) {
      difference = widget.selectedDate.difference(DateTime.now()).inDays;
    } else if (widget.typeOfCalendar == typesofCalendar.minutes) {
      difference = widget.selectTime - (DateTime.now().minute);
    } else if (widget.typeOfCalendar == typesofCalendar.hours) {
      difference = widget.selectTime - (DateTime.now().hour);
    }
    final width = widget.containerWidth / 2;
    offsetTemp = -1 * (width - (widget.width / 2) - 10);
    offsetTemp = - widget.initalOffset;

    super.initState();
  }
  @override
  Widget build(BuildContext context) {

    //DISMISS THE PICKER AFTER SELECTION
    void popView() {
      if (context != null) {
        if (popUpIndex == 0) {
          popUpIndex = popUpIndex + 1;
          Navigator.of(context).pop();
        }
      }
    }


    // TIME SELECTION METHOD
    void timeSelection(int index) {
      if (context != null) {
        if (index < 0) {
          index = index * (-1);
        }
        int clickedDate = index;
        if (DateTime.now().hour == index) {
          if (widget.typeOfCalendar == typesofCalendar.hours) {
            SharedClass.selectedHours = '${DateTime.now().hour.toString()} h';
            widget.timeCallBack('${SharedClass.selectedHours} ${SharedClass.selectedMinutes}');
          } else {
            SharedClass.selectedMinutes = '${DateTime.now().minute.toString()} m';
            widget.timeCallBack('${SharedClass.selectedHours} ${SharedClass.selectedMinutes}');
          }
        } else {
          if (widget.typeOfCalendar == typesofCalendar.hours) {
            SharedClass.selectedHours = '${clickedDate.toString()} h';
            widget.timeCallBack('${SharedClass.selectedHours} ${SharedClass.selectedMinutes}');
          } else {
            SharedClass.selectedMinutes = '${clickedDate.toString()} m';
            widget.timeCallBack('${SharedClass.selectedHours} ${SharedClass.selectedMinutes }');
          }
        }
        popView();

        controllerA.animateTo(finalValue.toDouble(),
            duration: Duration(seconds: 4), curve: Curves.fastLinearToSlowEaseIn);

         popView();
        Future.delayed(const Duration(seconds: 4), () { if (widget.typeOfCalendar == typesofCalendar.minutes) {
          SharedClass.minuteIndex.add(index);
        } else {
          SharedClass.hoursIndex.add(index);
        }});

      }
    }

    // DATE SELECTION METHOD
    void itemSelection(int index) {
      DateTime clickedDate = widget.selectedDate.add(Duration(days: index));
      if (DateTime.now() == DateTime.now().add(Duration(days: index))) {
        widget.clickCallBack(DateTime.now());
      } else {
        widget.clickCallBack(clickedDate);
      }

      _infiniteController.animateTo(finalValue,
          duration: Duration(seconds: 4), curve: Curves.fastLinearToSlowEaseIn);
      popView();
      Future.delayed(const Duration(seconds: 4), () { SharedClass.imgStream.add(clickedDate);});  }

    //SET POP UP VIEW OF THE WIDGET AFTER SELECTING THE DATE
    Widget dateList() {
      return InfiniteListView.builder(
          controller: _infiniteController,
          scrollDirection: Axis.horizontal,
          itemBuilder: (context2, index) {
            var currentDate = widget.selectedDate.add(Duration(days: index));
            return GestureDetector(
                onTap: () {
                  var offset = _infiniteController.offset;
                  if (index == 0) {
                    finalValue = offset + index * widget.width;
                    finalValue = offsetTemp;
                  } else if (index < 0) {
                    var indexDifference = index - selectedSetIndex;
                    var sizeValue = indexDifference * widget.width;
                    finalValue = offset + sizeValue;
                  }else if (index > 0) {
                    var indexDifference = index + selectedSetIndex;
                    var sizeValue = indexDifference * widget.width;
                    finalValue = offset + sizeValue;
                  }
                  Future.delayed(Duration(microseconds: 1000), () {
                    itemSelection(index);
                  });
                  selectedSetIndex = index;
                },
                child: Container(
                    child: DateCardView(
                        widget.selectedTextColor,
                        widget.selectedBackColor,
                        currentDate,
                        widget.width,
                        widget.height)));
          });
    }

    //DISPLAY TIME IN THE PICKER USING LIST
    Widget timeList() {
      return InfiniteListView.builder(
          controller: controllerA,
          itemCount: widget.typeOfCalendar == typesofCalendar.hours
              ? widget.is24Hours
              ? 24
              : 12
              : 60,
          scrollDirection: Axis.horizontal,
          itemBuilder: (context2, index) {
            var finalIndex = 0;
            finalIndex = index;
            if (finalIndex < 0) {
              finalIndex = finalIndex * (-1);
            }
            var currentDate = widget.selectTime + finalIndex;
            return GestureDetector(
                onTap: () {
                  var offset = controllerA.offset;
                  if (index == 0) {
                    finalValue = offset + index * widget.width;
                    finalValue = offsetTemp;
                  } else {
                    var indexDifference = index - selectedSetIndex;
                    var sizeValue = indexDifference * widget.width;
                    finalValue = offset + sizeValue;
                  }
                  Future.delayed(Duration(microseconds: 1000), () {
                    timeSelection(currentDate);
                  });
                  selectedSetIndex = index;
                },
                child: Container(
                    child: TimerCardView(
                        widget.selectedTextColor,
                        widget.selectedBackColor,
                        currentDate,
                        widget.width,
                        widget.height)));
          });
    }

    _infiniteController = InfiniteScrollController(
        initialScrollOffset: lerpDouble(0.0, offsetTemp, 1.0));
    controllerA = InfiniteScrollController(
        initialScrollOffset: lerpDouble(0.0, offsetTemp, 1.0));
    return Material(
        child: Column(children: [
          Container(
              height: widget.containerHeight,
              color: widget.listBackColor,
              padding: EdgeInsets.all(5),
              child: Container(
                child: widget.typeOfCalendar == typesofCalendar.days
                    ? dateList()
                    : timeList(),
              )),
        ]));

    // TIME LIST VIEW

    // DATE LIST VIEW
  }
}
